module arbiter_fair(
    input            clk, rstb,
    input      [1:0] r,
    output reg [1:0] g
);

// state
localparam [1:0] WAITR0 = 2'b00,
                 WAITR1 = 2'b01,
                 GRANT0 = 2'b10,
                 GRANT1 = 2'b11;

reg [1:0] state_reg, state_next;

// state register
always @(posedge clk or negedge rstb) begin
    if (~rstb) state_reg <= 0;
    else       state_reg <= state_next;
end

// next-state logic and output logic
always @* begin
    state_next = state_reg;
    g = 2'b00;
    case (state_reg)
        WAITR0: begin
            if (r[0]) state_next = GRANT0;
            else if (r[1]) state_next = GRANT1;
        end
        WAITR1: begin
            if (r[1]) state_next = GRANT1;
            else if (r[0]) state_next = GRANT0;
        end
        GRANT0: begin
            if (~r[0]) state_next = WAITR1;
            g[0] = 1'b1;
        end
        GRANT1: begin
            if (~r[1]) state_next = WAITR0;
            g[1] = 1'b1;
        end
        default: state_next = WAITR0;
    endcase
end

endmodule