module divider
#(
    parameter N = 8,
              CBIT = $clog2(N)
) (
    input              clk, rstb,
    input              start,
    input      [N-1:0] dvsr, dvnd,
    output reg         ready, done,
    output reg [N-1:0] quo, rmd
);

// state
localparam [1:0] IDLE = 2'b00,
                 OP   = 2'b01,
                 LAST = 2'b10,
                 DONE = 2'b11;

reg [1:0]      state_reg, state_next;
reg [N-1:0]    rh_reg, rh_next, rl_reg, rl_next, rh_tmp;
reg [N-1:0]    d_reg, d_next;
reg [CBIT-1:0] n_reg, n_next;
reg            qbit;

// state register
always @(posedge clk or negedge rstb) begin
    if (~rstb) begin
        state_reg <= 0;
        rh_reg    <= 0;
        rl_reg    <= 0;
        d_reg     <= 0;
        n_reg     <= 0;
    end
    else begin
        state_reg <= state_next;
        rh_reg    <= rh_next;
        rl_reg    <= rl_next;
        d_reg     <= d_next;
        n_reg     <= n_next;
    end
end

// next-state logic and output logic
always @* begin
    state_next = state_reg;
    rh_next = rh_reg;
    rl_next = rl_reg;
    d_next = d_reg;
    n_next = n_reg;
    ready = 1'b0;
    done = 1'b0;
    case (state_reg)
        IDLE: begin
            ready = 1'b1;
            if (start) begin
                rh_next = 0;
                rl_next = dvnd;
                d_next = dvsr;
                n_next = N;
                state_next = OP;
            end
        end
        OP: begin
            rl_next = {rl_reg[N-2:0], qbit};
            rh_next = {rh_tmp[N-2:0], rl_reg[N-1]};
            n_next = n_reg - 1;
            if (n_next==0) 
                state_next = LAST;
        end
        LAST: begin
            rl_next = {rl_reg[N-2:0], qbit};
            rh_next = rh_tmp;
            state_next = DONE;
        end
        DONE: begin
            done = 1'b1;
            state_next = IDLE;
        end
        default: state_next = IDLE;
    endcase
    quo = rl_reg;
    rmd = rh_reg;
end

// compare and subtract
always @* begin
    if (rh_reg>=d_reg) begin
        rh_tmp = rh_reg - d_reg;
        qbit = 1;
    end
    else begin
        rh_tmp = rh_reg;
        qbit = 0;
    end
end

endmodule