module fibonacci(
    input             clk, rstb,
    input             start,
    input       [4:0] i,
    output reg        ready, done,
    output reg [19:0] f
);

// state
localparam [1:0] IDLE = 2'b00,
                 OP   = 2'b01,
                 DONE = 2'b10;

reg  [1:0] state_reg, state_next;
reg [19:0] t0_reg, t1_reg, t0_next, t1_next;
reg  [4:0] n_reg, n_next;

// state register
always @(posedge clk or negedge rstb) begin
    if (~rstb) begin
        state_reg <= 0;
        t0_reg    <= 0;
        t1_reg    <= 0;
        n_reg     <= 0;
    end
    else begin
        state_reg <= state_next;
        t0_reg    <= t0_next;
        t1_reg    <= t1_next;
        n_reg     <= n_next;
    end
end

// next-state logic and output logic
always @* begin
    state_next = state_reg;
    t0_next = t0_reg;
    t1_next = t1_reg;
    n_next = n_reg;
    ready = 1'b0;
    done = 1'b0;
    case (state_reg)
        IDLE: begin
            ready = 1'b1;
            if (start) begin
                t0_next = 0;
                t1_next = 1;
                n_next = i;
                state_next = OP;
            end
        end
        OP: begin
            if (n_reg==0) begin
                t1_next = 0;
                state_next = DONE;
            end
            else if (n_reg==1)
                state_next = DONE;
            else begin
                t1_next = t1_reg + t0_reg;
                t0_next = t1_reg;
                n_next = n_reg - 1;
            end
        end
        DONE: begin
            done = 1'b1;
            state_next = IDLE;
        end
        default: state_next = IDLE;
    endcase
    f = t1_reg;
end

endmodule